# A RubyGem for creating mailing, shipping, address, barcode and other labels and rendering as PDF's.
#
# Author::    Aaron Wright  (mailto:acwrightdesign@gmail.com)
# Copyright:: Copyright (c) 2012 Infinite Token LLC
# License::  MIT License

require 'labels/document'
require 'labels/paper'
require 'labels/template'
require 'labels/database'
require 'labels/shape'
require 'labels/text'
require 'labels/image'
require 'labels/barcode'
require 'labels/date'

require 'prawn'

module Labels
  module PDF
    
    # This class builds a PDF document from the given Labels::Document
    class Builder
      
      # The document instance
      attr_accessor :document
      
      # Creates a new Labels::PDF::Builder instance
      def initialize(document=nil)
        @document = document
      end
      
      # Renders the PDF and optionally saves to given filename
      def render(filename=nil)
        unless filename.nil?
          self.pdf.render_file filename
        else
          self.pdf.render
        end
      end
      
      # Returns the rendered PDF document
      def pdf
        unless @document.nil?
          paper = @document.paper
          template = @document.template
          database = @document.database
        
          if paper && template
            document_options = {
              :page_size => [paper.width, paper.height],
              :left_margin => template.left_margin,
              :right_margin => template.right_margin,
              :top_margin => template.top_margin,
              :bottom_margin => template.bottom_margin
            }
            
            page_cell_count = template.columns * template.rows
            page_number_count = (database.data.length / page_cell_count).ceil
            cell = 0
            
            if page_number_count == 0
              page_number_count += 1
            end
            
            Prawn::Document.new document_options do |pdf|
              page_number_count.times do |page|
                if page > 0
                  pdf.start_new_page
                end
                
                pdf.define_grid({
                  :columns => template.columns,
                  :rows => template.rows,
                  :column_gutter => template.vertical_gutter,
                  :row_gutter => template.horizontal_gutter
                })
                
                # Draw template
                pdf.grid.rows.times do |i|
                  pdf.grid.columns.times do |j|
                    box = pdf.grid(i,j)
                    pdf.bounding_box box.top_left, :width => box.width, :height => box.height do
                      _draw_template(pdf, template)
                    end
                  end
                end
                
                # Draw layers
                pdf.grid.rows.times do |i|
                  pdf.grid.columns.times do |j|
                    box = pdf.grid(i,j)
                    pdf.bounding_box box.top_left, :width => box.width, :height => box.height do
                      layers = @document.layers.sort {|x,y| x.z <=> y.z}
                      layers.each do |layer|
                        
                        draw = false
                        content = nil
                        if database.data.length > 0
                          if cell < database.data.length
                            if database.data[cell].has_key?(layer.column)
                              content = database.data[cell][layer.column]
                            end
                            draw = true
                          end
                        else
                          draw = true
                        end
                        
                        if draw
                          case layer
                          when Labels::Barcode
                            _draw_barcode(pdf, layer, content)
                          when Labels::Date
                            _draw_date(pdf, layer, content)
                          when Labels::Image
                            _draw_image(pdf, layer, content)
                          when Labels::Shape
                            _draw_shape(pdf, layer, content)
                          when Labels::Text
                            _draw_text(pdf, layer, content)
                          end
                        end
                      end
                    end
                    cell += 1
                  end
                end
              end
            end
          end
        end
      end
        
      private
      
      # Draws the given barcode layer with optional content
      def _draw_barcode(pdf, layer, content=nil)
        if pdf.kind_of?(Prawn::Document) && layer.visible
          pdf.save_graphics_state

          origin = {:x => layer.x, :y => pdf.bounds.top - layer.y}
          center = {:x => origin[:x] + (layer.width * 0.5), :y => origin[:y] - (layer.height * 0.5)}

          pdf.rotate(360 - layer.rotation, :origin => [center[:x], center[:y]]) do 
            pdf.transparent layer.alpha do
              begin
                options = {
                  :bar_width => layer.bar_width,
                  :bar_height => layer.bar_height,
                  :color => layer.color,
                  :caption_height => layer.caption_height,
                  :caption_size => layer.caption_size,
                  :captioned => layer.captioned
                }
                
                unless content.nil?
                  options[:data] = content
                else
                  options[:data] = layer.content
                end
                  
                barcode = Barcodes.create(layer.symbology, options)
                renderer = Barcodes::Renderer::Pdf.new(barcode)

                barcode_width = ((barcode.width * 0.001) * 72.0)
                barcode_height = ((barcode.height * 0.001) * 72.0)

                pdf.bounding_box [origin[:x], origin[:y]], :width => layer.width, :height => layer.height do
                  renderer.draw(pdf)
                end
              rescue
                # Barcode error... render nothin
              end
            end
          end

          pdf.restore_graphics_state
        end
      end

      # Draws the given date layer with optional content
      def _draw_date(pdf, layer, content=nil)
        if content.nil?
          content = layer.content
        end

        begin
          content = Time.parse(content).strftime(layer.format)
        rescue
          content = Time.now.strftime(layer.format)
        end

        _draw_text(pdf, layer, content)
      end
      
      # Draws the given image layer with optional content
      def _draw_image(pdf, layer, content=nil)
        if pdf.kind_of?(Prawn::Document) && layer.visible
          pdf.save_graphics_state

          origin = {:x => layer.x, :y => pdf.bounds.top - layer.y}
          center = {:x => origin[:x] + (layer.width * 0.5), :y => origin[:y] - (layer.height * 0.5)}

          pdf.rotate(360 - layer.rotation, :origin => [center[:x], center[:y]]) do 
            pdf.transparent layer.alpha do
              begin
                unless content.nil?
                  content = Base64.decode64(content)
                else
                  content = Base64.decode64(layer.content)
                end

                temp = Tempfile.new(UUID.generate)
                temp.write(content)
                temp.rewind

                options = {
                  :at => [origin[:x], origin[:y]]
                }
                if layer.fit
                  options[:fit] = [layer.width, layer.height]
                end
                if layer.stretch
                  options[:width] = layer.width
                  options[:height] = layer.height
                end
                pdf.image temp, options

                temp.close
                temp.unlink
              rescue
                # Image error... render nothin
              end
            end
          end

          pdf.restore_graphics_state
        end
      end
      
      # Draws the given shape layer with optional content
      def _draw_shape(pdf, layer, content=nil)
        if pdf.kind_of?(Prawn::Document) && layer.visible
          pdf.save_graphics_state

          pdf.fill_color layer.fill_color
          pdf.stroke_color layer.stroke_color
          pdf.line_width layer.stroke_weight

          origin = {:x => layer.x, :y => pdf.bounds.top - layer.y}
          center = {:x => origin[:x] + (layer.width * 0.5), :y => origin[:y] - (layer.height * 0.5)}

          pdf.rotate(360 - layer.rotation, :origin => [center[:x], center[:y]]) do 
            pdf.transparent layer.alpha do
              unless layer.elliptical
                pdf.rounded_rectangle([origin[:x], origin[:y]], layer.width, layer.height, layer.corner_radius)
              else
                pdf.ellipse([center[:x], center[:y]], layer.width * 0.5, layer.height * 0.5)
              end

              pdf.fill_and_stroke if layer.fill && layer.stroke
              pdf.stroke if layer.stroke
              pdf.fill if layer.fill
            end
          end

          pdf.restore_graphics_state
        end
      end
      
      # Draws the given template
      def _draw_template(pdf, template)
        if pdf.kind_of?(Prawn::Document) && template.visible
          pdf.save_graphics_state

          if template.visible
            pdf.stroke do
              unless template.elliptical
                pdf.rounded_rectangle(pdf.bounds.top_left, template.width, template.height, template.corner_radius)
              else
                x = pdf.bounds.left + (pdf.bounds.width * 0.5)
                y = pdf.bounds.bottom + (pdf.bounds.height * 0.5)
                pdf.ellipse([x, y], template.width * 0.5, template.height * 0.5)
              end
            end
          end

          pdf.restore_graphics_state
        end
      end
      
      # Draws the given text layer with optional content
      def _draw_text(pdf, layer, content=nil)
        if pdf.kind_of?(Prawn::Document) && layer.visible
          pdf.save_graphics_state

          pdf.font layer.font_family
          pdf.fill_color layer.font_color

          origin = {:x => layer.x, :y => pdf.bounds.top - layer.y}
          center = {:x => origin[:x] + (layer.width * 0.5), :y => origin[:y] - (layer.height * 0.5)}

          pdf.rotate(360 - layer.rotation, :origin => [center[:x], center[:y]]) do 
            pdf.transparent layer.alpha do
              style = :normal
              style = :bold if layer.bold
              style = :italic if layer.italic
              style = :bold_italic if layer.bold && layer.italic

              options = {
                :at => [origin[:x], origin[:y]],
                :width => layer.width, 
                :height => layer.height,
                :overflow => :truncate,
                :size => layer.font_size,
                :align => layer.justification.to_sym,
                :style => style
              }
              unless content.nil?
                pdf.text_box content, options
              else
                pdf.text_box layer.content, options
              end
            end
          end

          pdf.restore_graphics_state
        end
      end
    end
  end
end