# A RubyGem for creating mailing, shipping, address, barcode and other labels and rendering as PDF's.
#
# Author::    Aaron Wright  (mailto:acwrightdesign@gmail.com)
# Copyright:: Copyright (c) 2012 Infinite Token LLC
# License::  MIT License

require 'nokogiri'

module Labels
  module XML
    
    # This class builds an XML representation of the given Labels::Document
    class Builder
      
      # The document instance
      attr_accessor :document
      
      # Creates a new Labels::XML::Builder instance
      def initialize(document=nil)
        @document = document
      end
      
      # Returns the XML document
      def xml
        unless @document.nil?
          builder = Nokogiri::XML::Builder.new do |xml|
            xml.doc.create_internal_subset(
              'label',
              nil,
              'label.dtd'
            )
            xml.label {
              xml.paper(
                :title => @document.paper.title) {
                xml.width @document.paper.width
                xml.height @document.paper.height
              }
              xml.template(
                :title => @document.template.title, 
                :description => @document.template.description, 
                :manufacturer => @document.template.manufacturer,
                :category => @document.template.category,
                :visible => @document.template.visible) {
                xml.width @document.template.width
                xml.height @document.template.height
                xml.columns @document.template.columns
                xml.rows @document.template.rows
                xml.send(:'left-margin', @document.template.left_margin)
                xml.send(:'right-margin', @document.template.right_margin)
                xml.send(:'top-margin', @document.template.top_margin)
                xml.send(:'bottom-margin', @document.template.bottom_margin)
                xml.send(:'horizontal-gutter', @document.template.horizontal_gutter)
                xml.send(:'vertical-gutter', @document.template.vertical_gutter)
                xml.elliptical @document.template.elliptical ? 'true' : 'false'
                xml.send(:'corner-radius', @document.template.corner_radius)
              }
              xml.layers {
                @document.layers.each do |layer|
                  xml.layer(
                    :title => layer.title,
                    :type => layer.type,
                    :locked => layer.locked,
                    :visible => layer.visible) {
                    xml.x layer.x
                    xml.y layer.y
                    xml.z layer.z
                    xml.width layer.width
                    xml.height layer.height
                    xml.alpha layer.alpha
                    xml.rotation layer.rotation
                    xml.content layer.content
                    
                    case layer.type
                    when 'shape'
                      xml.elliptical layer.elliptical ? 'true' : 'false'
                      xml.send(:'corner-radius', layer.corner_radius)
                      xml.fill layer.fill ? 'true' : 'false'
                      xml.stroke layer.stroke ? 'true' : 'false'
                      xml.send(:'fill-color', layer.fill_color)
                      xml.send(:'stroke-color', layer.stroke_color)
                      xml.send(:'stroke-weight', layer.stroke_color)
                    when 'text'
                      xml.send(:'font-family', layer.font_family)
                      xml.send(:'font-size', layer.font_size)
                      xml.send(:'font-color', layer.font_color)
                      xml.bold layer.bold ? 'true' : 'false'
                      xml.italic layer.italic ? 'true' : 'false'
                      xml.underline layer.underline ? 'true' : 'false'
                      xml.justification layer.justification
                    when 'image'
                      xml.fit layer.fit ? 'true' : 'false'
                      xml.stretch layer.stretch ? 'true' : 'false'
                    when 'barcode'
                      xml.symbology layer.symbology
                      xml.send(:'bar-width', layer.bar_width)
                      xml.send(:'bar-height', layer.bar_height)
                      xml.send(:'color', layer.color)
                      xml.send(:'caption-height', layer.caption_height)
                      xml.send(:'caption-size', layer.caption_size)
                      xml.captioned layer.captioned ? 'true' : 'false'
                    when 'date'
                      xml.send(:'font-family', layer.font_family)
                      xml.send(:'font-size', layer.font_size)
                      xml.send(:'font-color', layer.font_color)
                      xml.bold layer.bold ? 'true' : 'false'
                      xml.italic layer.italic ? 'true' : 'false'
                      xml.underline layer.underline ? 'true' : 'false'
                      xml.justification layer.justification
                      xml.format layer.format
                    end
                  }
                end
              }
              xml.database(:title => @document.database.title) {
                @document.database.data.each do |row|
                  xml.row {
                    row.each do |key, value|
                      case value
                      when Fixnum
                        xml.key value, :name => key, :type => 'integer'
                      when Float
                        xml.key value, :name => key, :type => 'float'
                      when Time
                        xml.key value.iso8601, :name => key, :type => 'date'
                      when TrueClass
                        xml.key 'true', :name => key, :type => 'boolean'
                      when FalseClass
                        xml.key 'false', :name => key, :type => 'boolean'
                      else
                        xml.key value, :name => key, :type => 'string'
                      end
                    end
                  }
                end
              }
            }
          end
          builder.to_xml
        end
      end
    end
  end
end